//////////////////////////////////////////////////////////////////
//
// RT_Project_01.cpp - cpp file
//
// This file was generated using the RTX64 Application Template for Visual Studio.
//
// Created: 1/25/2021 11:12:47 
// User: test
//
//////////////////////////////////////////////////////////////////

#include "RT_Project_01.h"
#include "SystemInitialization.h"
#include "IOModule.h"
#include "AxisConfiguration.h"
#include "SingleAxisMotion.h"
#include "Homing.h"
#include "Cam.h"
#include "Gear.h"
#include "GroupConfiguration.h"
#include "GroupMotion.h"
#include "TouchProbe.h"
#include "DigitalSwitch.h"
#include "Log.h"

int _tmain(int argc, _TCHAR * argv[])
{
    //Start the KINGSTAR Subsystem.
    if (!StartKingstar())
    {
        RtPrintf("The KINGSTAR Subsystem is not started.\n");
        return -1;
    }

    else
        RtPrintf("The KINGSTAR Subsystem is started.\n");

    RtPrintf("\n");

    //Create an instance of the SubsystemStatus structure and get the state from it.
    SubsystemStatus Subsystem = { ecatOffline, ecatOffline, 0, 0, 0, {ecatOffline}, {ecatOffline}, {axisOffline} };
    GetStatus(&Subsystem, NULL);

    //Display the details of the EtherCAT network.
    RtPrintf("Number of Devices found: %d\n", Subsystem.SlaveCount);
    RtPrintf("Number of Axes found: %d\n", Subsystem.AxesCount);
    RtPrintf("Number of I/O found: %d\n", Subsystem.IOCount);
    RtPrintf("\n");

#if 0

    /* Chapter 2: System Initialization */

    int nRet = 0;
    int resolution = 0;
    SlaveStatus Sts = { 0 };

    //Get the detailed information from a specified slave using an ID.
    nRet = GetSlaveById(0, &Sts);
    RtPrintf("%s cycle time: %d\n", Sts.Name, Sts.CycleTime);

    //Get information from an axis.
    nRet = GetAxisByIndex(2, &Sts, &resolution, NULL, NULL);
    RtPrintf("%s Vendor ID: %d\n", Sts.Name, Sts.VendorId);

    //Get information from an I/O module.
    nRet = GetIOByIndex(3, &Sts);
    RtPrintf("%s input length: %d\n", Sts.Name, Sts.InputLength);
    RtPrintf("%s output length: %d\n", Sts.Name, Sts.OutputLength);

    RtPrintf("\n");

#endif

#if 0

    /* Chapter 3: IO Control */

    //Display the length of the inputs and outputs of all I/O modules.
    GetAllInputsOutputs();

    //Write TRUE to the fourth I/O module, offset zero.
    WriteReadOutput(3, 0, 20);

    //Read the value from the third I/O module, offset zero.
    ReadInputOutput(2, 0);

    //Write a value into a simulated I/O module, offset zero.
    ForceInput(6, 0, 8);

#endif

#if 0

    /* Chapter 4: Axis Configuration */

    int Axis = 0;

    //Display the information of a device.
    PrintDeviceInformation(Axis);

    //Display the information of an axis.
    PrintAxisInformation(Axis);

    //Display the motion states of an axis.
    PrintMotionStates(Axis);

    //Get the available control modes from an axis. 
    GetAvailableControlMode(Axis);

    //Set a control mode for an axis.
    SetControlMode(Axis);

    //Create a motion profile for an axis.
    MotionProfile(Axis);

    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(Axis);

    //Set a soft limit switch for an axis.
    SoftLimitSwitch(Axis);

    //Set a limit sensor for an axis.
    //We comment out this function because we have used SoftLimitSwitch.
    //LimitSensor(Axis);

    //Limit the direction of an axis.
    LimitDirection(Axis);

    //PID tuning.
    UpdatePID(Axis);

#endif

#if 0

    /* Chapter 5: Single-Axis Motion */

    int Axis = 0;

    //Display the information of devices.
    PrintDeviceInformation(Axis);

    //Set a motion profile.
    //You need to set a motion profile, or the minimum following error will be large.
    MotionProfile(Axis);

    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(Axis);

    //PID tuning.
    /*When you use velocity or torque modes, you must tune PID.
      In this tutorial we use modeMasterIntPos, which doesn't require PID,
      but you should offer PID values for your drive to get the best performance.*/
    UpdatePID(Axis);

    //Set the starting positions for an axis.
    RtPrintf("Starting positions:\n\n");
    SetAxisStartingPosition(Axis);

    /* Enable an axis. */
    EnableAxis(Axis);

    /* Absolute moves */
    MoveAbsolute(Axis, 10000);
    //Sleep(5);

    //Reset the starting position so you know how far ContinuousAbsolute runs.
    //RtPrintf("Starting positions:\n\n");
    //SetAxisStartingPosition(Axis);
    //MoveContinuousAbsolute(Axis, 12000);

    /* Relative moves */
    MoveRelative(Axis, 9000);
    //Sleep(5);

    //Reset the starting position so you know how far ContinuousRelative runs.
    //RtPrintf("Starting positions:\n\n");
    //SetAxisStartingPosition(Axis);
    //MoveContinuousRelative(Axis, 11000);

    /* Additive move */
    MoveAdditive(Axis, -7000);

    /* Test moves */
    //Jog(Axis);
    //Inch(Axis, 8000);
    //Velocity(Axis);

    /* Motion queue samples */
    //QueueSample01(Axis, 10000, -5000);
    //QueueSample02(Axis, -2000, 5000, -4000);

    /* Blending sample */
    //Blending(Axis, 2000, 4000);

    /* Stop an axis */
    //Halt(Axis);
    //Stop(Axis);

    /* Disable an axis. */
    DisableAxis(Axis);

#endif

#if 0

    /* Chapter 6: Homing */

    int Axis = 1;

    //Display the information of devices.
    PrintDeviceInformation(Axis);

    //Set a motion profile.
    //You need to set a motion profile, or the minimum following error will be large.
    MotionProfile(Axis);

    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(Axis);

    //PID tuning.
    /*When you use velocity or torque modes, you must tune PID.
      In this tutorial we use modeMasterIntPos, which doesn't require PID,
      but you should offer PID values for your drive to get the best performance.*/
    UpdatePID(Axis);

    /* Enable an axis. */
    EnableAxis(Axis);

    /* Home an axis. */
    //HomingLatch(Axis);
    //HomingSoft(Axis);
    //HomingSensor(Axis);
    //HomingSlave(Axis);
    HomingOnPosition(Axis);

    /* Disable an axis. */
    DisableAxis(Axis);

#endif

#if 0

    /* Chapter 7: Camming */

    //Declare indexes for axes and a cam table.
    int Master = 1;
    int Slave = 6;
    int Table = 0;
    double setPosition = 0;
    KsCommandStatus cam = { 0 };

    //Display the information of devices.
    PrintDeviceInformation(Master);
    PrintDeviceInformation(Slave);

    //Set motion profiles.
    //You need to set motion profiles, or the minimum following error will be large.
    MotionProfile(Master);
    MotionProfile(Slave);

    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(Master);
    PositionUnitConversion(Slave);

    //PID tuning.
    /*When you use velocity or torque modes, you must tune PID.
      In this tutorial we use modeMasterIntPos, which doesn't require PID,
      but you should offer PID values for your drive to get the best performance.*/
    UpdatePID(Master);
    UpdatePID(Slave);

    //Enable axes.
    EnableAxis(Master);
    EnableAxis(Slave);

    //Set a starting position for the master and slave axes so they can be synchronized quickly.
    SetAxisPositionOffset(Master, setPosition, FALSE, mcImmediately);
    SetAxisPositionOffset(Slave, setPosition, FALSE, mcImmediately);

    //Define a cam table.
    DefineCamTable(Table);

    //Link the master and slave.
    //We use camRampTime for the cam mode.
    cam = AssignCamSlave(Master, Slave, Table);

    //Move the master and display its position.
    MoveRelative(Master, 150);

    //Display the slave's position.
    GetAnAxisPosition(Slave);

    //Check whether the axes are synchronized.
    if (cam.InSync)
        RtPrintf("The master and slave axes are synchronized.\n\n");

    //Detach the slave axis from the master.
    RemoveCamSlave(Slave);

    //Disable axes.
    DisableAxis(Master);
    DisableAxis(Slave);

#endif

#if 0

    /* Chapter 8: Gearing */

    //Declare indexes for axes and a gear state.
    int Master = 2;
    int Slave = 6;
    double setPosition = 0;
    KsCommandStatus gear = { 0 };

    //Display the information of devices.
    PrintDeviceInformation(Master);
    PrintDeviceInformation(Slave);

    //Set motion profiles.
    //You need to set motion profiles, or the minimum following error will be large.
    MotionProfile(Master);
    MotionProfile(Slave);

    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(Master);
    PositionUnitConversion(Slave);

    //PID tuning.
    /*When you use velocity or torque modes, you must tune PID.
      In this tutorial we use modeMasterIntPos, which doesn't require PID,
      but you should offer PID values for your drive to get the best performance.*/
    UpdatePID(Master);
    UpdatePID(Slave);

    //Enable axes.
    EnableAxis(Master);
    EnableAxis(Slave);

    //Set a starting position for the master and slave axes.
    SetAxisPositionOffset(Master, setPosition, FALSE, mcImmediately);
    SetAxisPositionOffset(Slave, setPosition, FALSE, mcImmediately);
    printf("Master starting position:\n");
    GetAnAxisPosition(Master);
    printf("Slave starting position:\n");
    GetAnAxisPosition(Slave);

    BOOL Gear = TRUE;   //TRUE: Gear. FALSE: Gear in position.
    if (Gear)
    {
        RtPrintf("Make a velocity move to use Gear.\n\n");

        //Gear the axes.
        gear = AssignGearSlave(Master, Slave);
    }

    else
    {
        double MasterPosition = -80;
        double SlavePosition = -100;

        RtPrintf("Make a velocity move to use GearInPos.\n\n");

        //To use SetAxisGearInPos, we need to use MoveAxisVelocity to move the master a bit.
        MoveAxisVelocity(Master, 100, 1000, 1000, 10000, mcNegativeDirection, mcBuffered);

        Sleep(100);

        //Gear in position.
        gear = AssignGearInPosSlave(Master, Slave, MasterPosition, SlavePosition);
    }

    //Move the master and display its position and velocity.
    RtPrintf("Master axis:\n");

    /*Move the master again for the slave to follow its velocity.
      Remember not to set the master's velocity too high,
      or the slave may not be able to follow the master.*/
    MoveAxisVelocity(Master, 360, 3600, 3600, 360000, mcNegativeDirection, mcBuffered);
    Sleep(1000);

    //Display the master's position and velocity.
    RtPrintf("End position:\n");
    GetAnAxisPosition(Master);
    GetAnAxisVelocity(Master);
    RtPrintf("----------\n\n");

    //Display the slave's position and velocity.
    RtPrintf("Slave axis:\n");
    RtPrintf("End position:\n");
    GetAnAxisPosition(Slave);
    GetAnAxisVelocity(Slave);

    //Check whether the axes are synchronized.
    if (gear.InSync)
        RtPrintf("The master and slave axes are synchronized.\n\n");

    Halt(Master);

    //Detach the slave axis from the master.
    RemoveGearSlave(Slave);

    //Disable axes.
    DisableAxis(Master);
    DisableAxis(Slave);

#endif

#if 0

    /* Chapter 9: Group Configuration */

    //Display the information of devices.
    PrintDeviceInformation(0);
    PrintDeviceInformation(1);
    PrintDeviceInformation(2);
    PrintDeviceInformation(5);
    PrintDeviceInformation(6);

    //Set motion profiles.
    //You need to set motion profiles, or the minimum following error will be large.
    MotionProfile(0);
    MotionProfile(1);
    MotionProfile(2);
    MotionProfile(5);
    MotionProfile(6);

    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(0);
    PositionUnitConversion(1);
    PositionUnitConversion(2);
    PositionUnitConversion(5);
    PositionUnitConversion(6);

    //PID tuning.
    /*When you use velocity or torque modes, you must tune PID.
      In this tutorial we use modeMasterIntPos, which doesn't require PID,
      but you should offer PID values for your drive to get the best performance.*/
    UpdatePID(0);
    UpdatePID(1);
    UpdatePID(2);
    UpdatePID(5);
    UpdatePID(6);

    //Set all axes' starting positions to zero.
    SetAllAxesStartingPosition(7);

    //Add Axis Zero and One to Group Zero.
    AssignAxis(0, 0, 0);
    AssignAxis(0, 1, 1);
    AssignAxis(0, 2, 2);

    //Add Axis Five and Six to Group One.
    AssignAxis(1, 5, 0);
    AssignAxis(1, 6, 1);

    //Check which axes have been added to which group.
    RtPrintf("Check which axes have been added to which group:\n\n");

    CheckAxisInGroup(0, 0);
    CheckAxisInGroup(0, 1);
    CheckAxisInGroup(0, 2);
    CheckAxisInGroup(1, 0);
    CheckAxisInGroup(1, 1);

    //Set group motion profiles.
    MotionProfileGroup(0);
    MotionProfileGroup(1);

    //Enable the groups.
    GroupEnable(0);
    GroupEnable(1);

    //Get the groups' states.
    GetAGroupState(0);
    GetAGroupState(1);

    //Convert coordinates from ACS to MCS.
    ACSToMCS(0);

    //Convert coordinates from MCS to PCS.
    MCSToPCS(0);

    //Disable Group Zero.
    GroupDisable(0);

    //Remove the axes in Group Zero one by one.
    //Group Zero will be disabled after all axes are removed.
    RemoveAxis(0, 0);
    RemoveAxis(0, 1);

    //Remove all the axes in Group One at once.
    RemoveGroup(1);

    //Get the groups' states.
    GetAGroupState(0);
    GetAGroupState(1);

#endif

#if 0

    /* Chapter 10: Group Motion */

    const int ARRLENGTH = 3;
    int Axis[ARRLENGTH] = { 1, 5, 7 };

    //Display the information of devices.
    PrintDeviceInformation(Axis[0]);
    PrintDeviceInformation(Axis[1]);
    PrintDeviceInformation(Axis[2]);

    //Set motion profiles.
    //You need to set motion profiles, or the minimum following error will be large.
    MotionProfile(Axis[0]);
    MotionProfile(Axis[1]);
    MotionProfile(Axis[2]);

    //Set a group motion profile.
    //It is optional to use a group motion profile.
    MotionProfileGroup(0);

    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(Axis[0]);
    PositionUnitConversion(Axis[1]);
    PositionUnitConversion(Axis[2]);

    //PID tuning.
    /*When you use velocity or torque modes, you must tune PID.
      In this tutorial we use modeMasterIntPos, which doesn't require PID,
      but you should offer PID values for your drive to get the best performance.*/
    UpdatePID(Axis[0]);
    UpdatePID(Axis[1]);
    UpdatePID(Axis[2]);

    //Add axes to Group Zero.
    AssignAxis(0, Axis[0], 0);
    AssignAxis(0, Axis[1], 1);
    AssignAxis(0, Axis[2], 2);

    //Set the starting positions for axes.
    //SetGroupPositionOffset must be used when groups are disabled.
    RtPrintf("Starting positions:\n\n");
    SetGroupStartingPosition(0);

    //Display a group's state.
    GetAGroupState(0);

    //Enable a group.
    GroupEnable(0);

    //Display a group's state.
    GetAGroupState(0);

    //Reset and enable axes.
    EnableAxis(Axis[0]);
    EnableAxis(Axis[1]);
    EnableAxis(Axis[2]);

    //Move an axis group
    /* Linear moves */
    MoveLinearAbsoluteGroup(0);
    //MoveLinearRelativeGroup(0);
    //MoveLinearAdditiveGroup(0);

    /* Circular moves */
    //MoveCircularAbsoluteGroup(0);
    //MoveCircularRelativeGroup(0);
    //MoveCircularAdditiveGroup(0);

    /* Helical moves */
    //MoveHelicalAbsoluteGroup(0);
    //MoveHelicalRelativeGroup(0);
    //MoveHelicalAdditiveGroup(0);

    /* Test moves */
    //GroupJog(0);
    //GroupInch(0);

    /* Blending sample */
    //BlendSample(0);

    //Enable digital switches using a group's distance.
    /*The code of this function is in DigitalSwitch.cpp. Because it needs
      a group to run, it is put in Chapter 10's Main function.
      To learn how it works, see Chapter 12.*/
    //GroupCamSwitch(0);

    /* Stop a group */
    //GroupHalt(0);
    //GroupStop(0);

    //Disable Group Zero.
    GroupDisable(0);

    //Remove all the axes in Group Zero at once.
    RemoveGroup(0);

    //Disable axes.
    DisableAxis(Axis[0]);
    DisableAxis(Axis[1]);
    DisableAxis(Axis[2]);

#endif

#if 0

    /* Chapter 11: Touch Probe */

    int Axis = 2;

    //Display the information of devices.
    PrintDeviceInformation(Axis);

    //Set a motion profile.
    //You need to set a motion profile, or the minimum following error will be large.
    MotionProfile(Axis);

    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(Axis);

    //PID tuning.
    /*When you use velocity or torque modes, you must tune PID.
      In this tutorial we use modeMasterIntPos, which doesn't require PID,
      but you should offer PID values for your drive to get the best performance.*/
    UpdatePID(Axis);

    //Set the starting positions for an axis.
    RtPrintf("Starting positions:\n\n");
    SetAxisStartingPosition(Axis);

    //Enable an axis.
    EnableAxis(Axis);

    //Record a position using touch probe.
    RecordPosition(Axis);

    //Disable an axis.
    DisableAxis(Axis);

#endif

#if 0

    /* Chapter 12: Digital Switch */

    int Axis = 2;
    
    //Display the information of devices.
    PrintDeviceInformation(Axis);
    
    //Set a motion profile.
    //You need to set a motion profile, or the minimum following error will be large.
    MotionProfile(Axis);
    
    //Convert the unit. In this tutorial we convert it to degrees.
    PositionUnitConversion(Axis);
    
    //PID tuning.
    /*When you use velocity or torque modes, you must tune PID.
      In this tutorial we use modeMasterIntPos, which doesn't require PID,
      but you should offer PID values for your drive to get the best performance.*/
    UpdatePID(Axis);
    
    //Set the starting positions for an axis.
    RtPrintf("Starting positions:\n\n");
    SetAxisStartingPosition(Axis);
    
    //Enable an axis.
    EnableAxis(Axis);
    
    //Enable digital switches using an axis' position.
    DigitalCamSwitch(Axis);
    //DigitalCyclicSwitch(Axis);
    
    //Disable an axis.
    DisableAxis(Axis);
    
#endif

#if 0

    /* Chapter 13: Log */

    //Work with user variables.
    //CreateUserVariables();

    //Record the change of user variables.
    //RecordData();

    //Display user variables' values in Analysis Console.
    DebugMessages();

    //Use user variables between two projects.
    //LogProject();

#endif

    //Stop the KINGSTAR Subsystem.
    if (!StopKingstar())
    {
        RtPrintf("The KINGSTAR Subsystem is not stopped.\n");
        return -1;
    }

    else
        RtPrintf("The KINGSTAR Subsystem is stopped.\n");
  
    return 0;
}
